"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const HapNodeJS = __importStar(require("hap-nodejs"));
const version = require('../../package.json').version.trim();
module.exports = function (RED) {
    const debug = require('debug')('NRCHKB:api');
    const serviceData = {};
    const _initServiceAPI = () => {
        debug('Initialize ServiceAPI');
        Object.values(HapNodeJS.Service)
            .filter(service => service.prototype instanceof HapNodeJS.Service)
            .map(service => {
            const newService = HapNodeJS.Service.serialize(new service());
            newService.displayName = service.name;
            return newService;
        })
            .sort((a, b) => a.displayName < b.displayName ? -1 : a.displayName > b.displayName ? 1 : 0)
            .forEach(serialized => serviceData[serialized.displayName] = serialized);
        RED.httpAdmin.get('/nrchkb/service/types', RED.auth.needsPermission('homekit.read'), (_req, res) => {
            res.json(serviceData);
        });
    };
    const _initNRCHKBVersionAPI = () => {
        debug('Initialize NRCHKBVersionAPI');
        debug('Running version:', version);
        const releaseVersionRegex = /(\d+)\.(\d+)\.(\d+)/;
        const devVersionRegex = /(\d+)\.(\d+)\.(\d+)-dev\.(\d+)/;
        const releaseVersionFound = releaseVersionRegex.test(version);
        const devVersionFound = devVersionRegex.test(version);
        let xyzVersion = '0.0.0';
        if (devVersionFound) {
            try {
                const match = devVersionRegex.exec(version);
                if (match) {
                    xyzVersion = 0 + '.' + match[1] + match[2] + match[3] + '.' + match[4];
                }
                else {
                    debug('Could not match dev version');
                }
            }
            catch (e) {
                console.error(e);
            }
        }
        else if (releaseVersionFound) {
            try {
                const match = releaseVersionRegex.exec(version);
                if (match) {
                    xyzVersion = match[0];
                }
                else {
                    debug('Could not match release version');
                }
            }
            catch (e) {
                console.error(e);
            }
        }
        else {
            debug('Bad version format');
            xyzVersion = '0.0.0';
        }
        debug('Evaluated as:', xyzVersion);
        RED.httpAdmin.get('/nrchkb/version', RED.auth.needsPermission('homekit.read'), (_req, res) => {
            res.json({
                version: xyzVersion,
            });
        });
    };
    const init = () => {
        _initServiceAPI();
        _initNRCHKBVersionAPI();
    };
    return {
        init: init,
    };
};
